/**
 * Copyright (C) 2017 Wasabeef
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var RE = {};

RE.currentSelection = {
    "startContainer": 0,
    "startOffset": 0,
    "endContainer": 0,
    "endOffset": 0
};

RE.editor = document.getElementById('editor');
RE.editorTitle = document.getElementById('editor-title');

// Height of content window, will be set by viewController
RE.contentHeight = 200;

// 初始化
RE.init = function () {
    
    $(document).on('selectionchange',function(e){
            if (window.PubArticleJSInterface) {
            } else {
                // 只用安卓的才调用
                RE.calculateEditorHeightWithCaretPosition();
            }
          
    });
    
    $('.editor-title').focus(function () {
        if (window.PubArticleJSInterface) {
            window.PubArticleJSInterface.titleEditorFocus(true);
        } else {
            window.webkit.messageHandlers.titleEditorFocus.postMessage({});
        }
    });

    $('.editor-title').blur(function () {
        if (window.PubArticleJSInterface) {
            window.PubArticleJSInterface.titleEditorFocus(false);
        } else {
        }
    });

    $('#editor').focus(function () {
        if (window.PubArticleJSInterface) {
            window.PubArticleJSInterface.contentEditorFocus(true);
        } else {
            window.webkit.messageHandlers.contentEditorFocus.postMessage({});
        }
    });

    $('.editor-title').on('input', function () {
        var editorTitle = RE.editorTitle.innerText;
        if (editorTitle.length > 50) {
            // 字数保持20个  超长失去焦点 不可输入
            RE.setArticleTitle(editorTitle.substring(0, 50));
            RE.editorTitle.blur();
        } else {
            RE.cansaveArticle();
        };
    });

    $('#editor').on('input', function () {
        // 判断是否可以进行下一步
        RE.cansaveArticle();
    });

    // 点击网页上的其它位置 聚焦
    $(window).on('touchend', function (e) {
        if (e.target.id == "editor_footer" || e.target.nodeName.toLowerCase() == "html") {
            RE.focusEditor();
        }
    });


}

RE.setEditorHeight = function () {
    if (window.PubArticleJSInterface) {
        var editor = $('#editor')
        editor.css('min-height', 300)
    };
}

RE.calculateEditorHeightWithCaretPosition = function() {
    
    var padding = 0;
    var c = RE.getCaretYPosition();
    
    var editor = $('#editor');
    
    var offsetY = window.document.documentElement.scrollTop;
    var height = RE.contentHeight;
    var newPos = window.pageYOffset;
    
    if (c < offsetY) {
        newPos = c;
        window.scrollTo(0, newPos);
    } else if (c > (offsetY + height - padding)) {
        newPos = c - height + padding;
        window.scrollTo(0, newPos);
    }
    
}

RE.getCaretYPosition = function() {
    var sel = window.getSelection();
    // Next line is comented to prevent deselecting selection. It looks like work but if there are any issues will appear then uconmment it as well as code above.
    //sel.collapseToStart();
    var range = sel.getRangeAt(0);
    var span = document.createElement('span');// something happening here preventing selection of elements
    range.collapse(false);
    range.insertNode(span);
    var topPosition = span.offsetTop;
    span.parentNode.removeChild(span);
    return topPosition;
}

RE.cansaveArticle = function () {
    var editorTitle = RE.editorTitle.innerText;
    var editorText = RE.editor.innerText;
    var editorHtml = RE.editor.innerHTML;;
    if (window.PubArticleJSInterface) {
        window.PubArticleJSInterface.setTitleAndContent(editorTitle, editorText);
    } else if (window.webkit.messageHandlers) {
        window.webkit.messageHandlers.setTitleAndContent.postMessage({ 'title': editorTitle, 'content': editorText, 'htmlStr':editorHtml });
    }
}


document.addEventListener("selectionchange", function () {
    RE.backuprange();
});

// 设置标题title
RE.setArticleTitle = function (title) {
    $('.editor-title').text(title);
}

// 获取标题title
RE.articleTitle = function () {
    return RE.editorTitle.innerText;
}

// Initializations
RE.callback = function () {
    // window.location.href = "re-callback://" + encodeURI(RE.getHtml());
}

RE.setHtml = function (contents) {
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
    // 图片上添加删除节点
    $('.article-img').append('<img class="delete-button" src="editor_remove.png" />');
    RE.enabledEditingItems();
}

RE.getHtml = function () {
    // 先删除所有 delete 节点
    $('.delete-button').remove();
    var html = RE.editor.innerHTML;
    // 在插入回来
    $('.article-img').append('<img class="delete-button" src="editor_remove.png" />');
    RE.enabledEditingItems();
    return html;
}

RE.getHtmlWithoutLabel = function () {
    return RE.editor.innerText;
}

RE.getText = function () {
    return RE.editor.innerText;
}

// 同时获取文章标题 和 内容, 用于存草稿、发布
RE.getTitleAndContent = function () {
    var editorTitle = RE.editorTitle.innerText;
    var editorContent = RE.getHtml();
    return { 'editorTitle': editorTitle, 'editorContent': editorContent };
}

//去掉富文本编辑的html，样式，只显示纯文字内容
RE.removeTextFormat = function(str) {
    str = str.replace(/<\/?[^>]*>/g, "");
    return str;
}

RE.setBaseTextColor = function (color) {
    RE.editor.style.color = color;
}

RE.setBaseFontSize = function (size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function (left, top, right, bottom) {
    RE.editor.style.paddingLeft = left;
    RE.editor.style.paddingTop = top;
    RE.editor.style.paddingRight = right;
    RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function (color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function (image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function (size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function (size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function (align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function (align) {
    RE.editor.style.verticalAlign = align;
}

RE.setTitlePlaceholder = function (placeholder) {
//    alert(placeholder)
    var editor = $('#editor-title');
    //set placeHolder
    editor.attr("placeholder",placeholder);
    
    //set focus
    editor.focusout(function(){
        var element = $(this);
        if (!element.text().trim().length) {
            element.empty();
        }
    });
}

// 设置content placeHolderasds
RE.setContentPlaceholder = function (placeholder) {
//    alert(placeholder)
    var editor = $('#editor');
    //set placeHolder
    editor.attr("placeholder",placeholder);
    
    //set focus
    editor.focusout(function(){
        var element = $(this);
        var htmlString = RE.editor.innerHTML;;
        htmlString = htmlString.replace(/<br>/,'');
        if (!element.text().trim().length && !htmlString.trim().length) {
            element.empty();
        }
    });
}

RE.setInputEnabled = function (inputEnabled) {
    RE.editor.contentEditable = String(inputEnabled);
}

RE.undo = function () {
    document.execCommand('undo', false, null);
}

RE.redo = function () {
    document.execCommand('redo', false, null);
}

RE.setBold = function () {
    document.execCommand('bold', false, null);
}

RE.setItalic = function () {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function () {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function () {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function () {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function () {
    document.execCommand('underline', false, null);
}

RE.setBullets = function () {
    document.execCommand('insertUnorderedList', false, null);
}

RE.setNumbers = function () {
    document.execCommand('insertOrderedList', false, null);
}

RE.setTextColor = function (color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function (color) {
    RE.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setFontSize = function (fontSize) {
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function (heading) {
    document.execCommand('formatBlock', false, '<h' + heading + '>');
}

RE.setIndent = function () {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function () {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function () {
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function () {
    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function () {
    document.execCommand('justifyRight', false, null);
}

RE.setBlockquote = function () {
    document.execCommand('formatBlock', false, '<blockquote>');
}

// 插入图片
RE.insertImage = function (url, alt) {
    //    RE.restorerange();
    var current_selection = $(RE.getSelectedNode());

    var html = '<p class="article-img"><img src="' + url + '" style="width:100%;height:auto;"/><img class="delete-button" src="editor_remove.png" /></p>';
    if (!current_selection.text().length || !$(current_selection).prev()) {
        // 如果当前选中节点无文案 且 无前兄弟节点
        html = '<br/>' + html;
    }

    if (RE.currentSelection.startOffset < current_selection.text().length || $(current_selection).next().text().length > 0) {
        // 如果光标后面还有文字，则不插入后置 <br> 或者下一节点 text 则不插入 <br>
    } else {
        html = html + '<p><br/></p>';
    }

    RE.insertHTML(html);
    RE.enabledEditingItems(); // 设置图片可编辑
    document.activeElement.blur(); // 插入图片后 失去焦点
}

RE.insertHTML = function (html) {
    RE.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function (url, title) {
    RE.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML", false, "<a href='" + url + "'>" + title + "</a>");
    } else if (sel.rangeCount) {
        var el = document.createElement("a");
        el.setAttribute("href", url);
        el.setAttribute("title", title);

        var range = sel.getRangeAt(0).cloneRange();
        range.surroundContents(el);
        sel.removeAllRanges();
        sel.addRange(range);
    }
    RE.callback();
}

RE.setTodo = function (text) {
    var html = '<input type="checkbox" name="' + text + '" value="' + text + '"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.prepareInsert = function () {
    RE.backuprange();
}

RE.backuprange = function () {
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
        var range = selection.getRangeAt(0);
        RE.currentSelection = {
            "startContainer": range.startContainer,
            "startOffset": range.startOffset,
            "endContainer": range.endContainer,
            "endOffset": range.endOffset
        };
    }
}

RE.restorerange = function () {
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

RE.enabledEditingItems = function (e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    // window.location.href = "re-state://" + encodeURI(items.join(','));
}

RE.focus = function () {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function () {
    RE.editor.blur();
}

RE.removeFormat = function () {
    document.execCommand('removeFormat', false, null);
}

// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function (e) {
    var KEY_LEFT = 37, KEY_RIGHT = 39;
    if (e.which == KEY_LEFT || e.which == KEY_RIGHT) {
        RE.enabledEditingItems(e);
    }
});
RE.editor.addEventListener("click", RE.enabledEditingItems);

// 获取当前选中的节点
RE.getSelectedNode = function () {
    var node, selection;
    if (window.getSelection) {
        selection = getSelection();
        node = selection.anchorNode;
    }
    if (!node && document.selection) {
        selection = document.selection
        var range = selection.getRangeAt ? selection.getRangeAt(0) : selection.createRange();
        node = range.commonAncestorContainer ? range.commonAncestorContainer :
            range.parentElement ? range.parentElement() : range.item(0);
    }
    if (node) {
        return (node.nodeName == "#text" ? node.parentNode : node);
    }
};

// 回车插入 p 标签
RE.returnTap = function (evt) {
    evt = (evt) ? evt : ((window.event) ? window.event : "");
    keyCode = evt.keyCode ? evt.keyCode : (evt.which ? evt.which : evt.charCode);
    if (keyCode === 13) {
        //            document.execCommand('formatBlock', false, '<p>');
        //            window.getSelection().focusNode.parentNode.setAttribute("class","oder2");
        var current_selection = $(RE.getSelectedNode());
        var t = current_selection.prop("tagName").toLowerCase();
        var is_heading = (t == 'h1' || t == 'h2' || t == 'h3' || t == 'h4' || t == 'h5' || t == 'h6');
        if (!is_heading) {
            document.execCommand('formatBlock', false, '<p>');
            //            var listId = window.getSelection().focusNode.parentNode;
            //            $(listId).addClass("oder2");
        }
    }
}

// 设置自动滚动
RE.updateOffset = function (height) {
    window.scrollTo(0, height);
}

// 设置footer高度
RE.setFooterHeight = function (footerHeight) {
    var footer = $('#editor_footer');
    footer.height(footerHeight + 'px');
}

// 设置可编辑
RE.enabledEditingItems = function (e) {
    // Images 不可编辑
    $('.article-img').attr('contenteditable', false);
    $('.article-img').on('click', function (event) {
        document.activeElement.blur();
    });

    $('.delete-button').on('click', function (event) {
        document.activeElement.blur();
        $(this).parents('.article-img').remove();
        if (window.PubArticleJSInterface) {
            window.PubArticleJSInterface.deleteImageForStatistics();
        } else {
            window.webkit.messageHandlers.deleteImageForStatistics.postMessage({});
        }
    });
}

// 自动聚焦点
RE.focusEditor = function (e) {
    var titleEditor = $('#editor');
    var range = document.createRange();
    range.selectNodeContents(titleEditor.get(0));
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    titleEditor.focus();
}

